#!/usr/bin/python
import ujson
import os
import sys

PIPELINE_SET_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/pipeline.json'
VIDEOSOURCE_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/pipeline/0/videosource/0.json'
MG_JNODE_FILEPATH = '/etc/edge/mediagrabber.0/main.conf'

RAMDISK_DIRPATH = '/var/spool/edge'

CONST_300M = 314572800
CONST_250M = 262144000


with open(PIPELINE_SET_JNODE_FILEPATH, 'r') as json_file:
    pipelines_jnode = ujson.load(json_file)

# if len(pipelines_jnode) != 1:
#     print "Number of pipelines != 1"
#     sys.exit(1)

with open(VIDEOSOURCE_JNODE_FILEPATH, 'r') as json_file:
    videosource_jnode = ujson.load(json_file)

if videosource_jnode['rw']['type'] != 'mv':
    print "Not a MV pipeline"
    sys.exit(1)

if len(sys.argv) == 2 and sys.argv[1] in ('-h', '--help'):
    print "Usage: \n\tmediastore-cleanup-args.py <image_width> <image_height> <image_fps> <file_count>"
    sys.exit(0)

with open(MG_JNODE_FILEPATH, 'r') as json_file:
    mediagrabber_jnode = ujson.load(json_file)

image_width = mediagrabber_jnode['Driver']['ParamSet']['VideoSource']['Acquisition']['Sensor']['Width']
image_height = mediagrabber_jnode['Driver']['ParamSet']['VideoSource']['Acquisition']['Sensor']['Width']
file_count = mediagrabber_jnode['FileApi']['FileCountHigh']
image_fps = mediagrabber_jnode['Driver']['ParamSet']['VideoSource']['Acquisition']['Fps']

statvfs = os.statvfs(RAMDISK_DIRPATH)
ramdisk_size = statvfs.f_frsize * statvfs.f_blocks

mg_store_size = (image_height * image_width) * file_count

du_max_mb = (ramdisk_size - mg_store_size) - CONST_300M
du_min_mb = du_max_mb - CONST_250M

du_max_perc = float(du_max_mb) / ramdisk_size
du_min_perc = float(du_min_mb) / ramdisk_size

result_jnode = {
    'Cleanup': {
        'CheckPeriod': image_fps,
        'DiskLoUsage': round(du_min_perc, 2),
        'DiskHiUsage': round(du_max_perc, 2),
        'InodeLoMaxUsage': 0.7,
        'InodeHiMaxUsage': 0.8
    },
    'CleanupMode': 2
}

print ujson.dumps(result_jnode, indent=2)
