import os
import sys
import ujson

from twisted.internet import reactor
from edge.argset import ArgSet
from edge import eventlog

from _args import KernelArgs
from _kernel import Kernel
from __version import VERSION_STRING


def run_module(raw_arg_set):
    """
    :type raw_arg_set: list[str]
    """
    arg_set = ArgSet(raw_arg_set)
    if arg_set.mode == ArgSet.MODE_NONE:
        print "The command line's arguments are invalid"
        sys.exit(1)

    if arg_set.mode == ArgSet.MODE_VERSION:
        print VERSION_STRING
        return

    if arg_set.mode == ArgSet.MODE_STARTUP:
        module_conf_name = 'overlayrender.{}'.format(arg_set.module_id)
        eventlog.open_log(module_conf_name, arg_set.log_level)
        eventlog.log_i('Startup...')
        try:
            main_conf_abspath = os.path.join('/etc/edge', module_conf_name, 'main.conf')
            with open(main_conf_abspath) as main_conf_jnode:
                main_conf_jnode = ujson.load(main_conf_jnode)

            args = KernelArgs.unpack(main_conf_jnode, arg_set.log_level)
            kernel = Kernel(reactor, args)
            kernel.run()

        finally:
            eventlog.log_i('Exit')
            eventlog.close_log()
