import ujson
import os

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *
from os.path import join as join_path

import edge.pipeline_helpers as helpers


def gc_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    # speed
    if ui_entry_point in [Pipeline.Speed.Ui.VIDEOSOURCE_0]:
        used_ui_modules.add(Pipeline.Speed.Ui.VIDEOSOURCE_0)
        used_edge_modules.add(Pipeline.Speed.Edge.GRAB_CONTROL_0)
        creator_ctor = GcCreator.create

    elif ui_entry_point in [Pipeline.Speed.Ui.PIPELINE]:
        used_ui_modules.add(Pipeline.Speed.Ui.VIDEOSOURCE_0)
        creator_ctor = GcCreator.create

    # mv
    elif ui_entry_point in [Pipeline.Mv.Ui.VIDEOSOURCE_0]:
        used_ui_modules.add(Pipeline.Mv.Ui.VIDEOSOURCE_0)
        used_edge_modules.add(Pipeline.Mv.Edge.GRAB_CONTROL_0)
        creator_ctor = GcCreator.create

    elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE]:
        used_ui_modules.add(Pipeline.Mv.Ui.VIDEOSOURCE_0)
        creator_ctor = GcCreator.create

    return creator_ctor


class GcCreator(BaseCreator):
    def __init__(self, id_main_setupstore):
        """
        :type id_main_setupstore: str, dict, str, dict
        """
        BaseCreator.__init__(self)
        self._id_main_setupstore = id_main_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: RgCreator
        """
        if ui_entry_point in [Pipeline.Speed.Ui.PIPELINE, Pipeline.Speed.Ui.VIDEOSOURCE_0]:
            id_main_setupstore = GcCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = GcCreator(id_main_setupstore)

        elif ui_entry_point in [Pipeline.Mv.Ui.PIPELINE, Pipeline.Mv.Ui.VIDEOSOURCE_0]:
            id_main_setupstore = GcCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map)
            creator = GcCreator(id_main_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :rtype: str, dict, str
        """
        edge_group = helpers.get_edge_group(ui_pid)
        spool_dir = get_spool_dir(ui_pid)

        if ui_entry_point.ptype == PTYPE_SPEED:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Speed.Ui.VIDEOSOURCE_0]
            edge_gid = Pipeline.Speed.Edge.GRAB_CONTROL_0.gid
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.GRAB_CONTROL_0)

        elif ui_entry_point.ptype == PTYPE_LPR_MV:
            ui_videosource_jnode = ui_identity_to_jnode_map[Pipeline.Mv.Ui.VIDEOSOURCE_0]
            edge_gid = Pipeline.Mv.Edge.GRAB_CONTROL_0.gid
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.GRAB_CONTROL_0)
        else:
            raise NotImplemented

        ui_acquisition_jnode = ui_videosource_jnode['rw']['videostream']

        ui_control_jnode = ui_acquisition_jnode['control']

        #
        exp_time_conf = [0, 0, 0]
        gain_conf = [0, 0, 0]
        gain_step = 1.0
        exptime_step = 20
        if ui_control_jnode['profile'] == 'auto-70-150':
            gain_conf = [ui_control_jnode['gain-auto-min'], ui_control_jnode['gain-auto-max'], gain_step]
            exp_time_conf = [ui_control_jnode['exposure-auto-min'], ui_control_jnode['exposure-auto-max'], exptime_step]
        #

        main_conf = {
            "Analyzer": {
                "ParamSet": {
                    "Target": {
                        "Median": [70, 150],
                        "SteadyState": {
                            "Unstable": 5,
                            "PreStable": 15,
                            "Stable": 30
                        }
                    },
                    "Sweep": {
                        "Gain": gain_conf,
                        "ExpTime": exp_time_conf,
                        "Gamma": [0.4, 1.5, 0.1],
                        "Lut12": {
                            "Default": "/opt/edge/share/setupstore/edge/mediagrabber/lut12.default.conf",
                            "Evening": "",
                            "Night": "/opt/edge/share/setupstore/edge/mediagrabber/lut12.olvia.conf",
                        }
                    }
                }
            },
            "FileApi": {
                "AddSceneImgReact":
                    {
                        "Url": join_path(spool_dir, 'mg%s-gc%s-scene' % (edge_gid, edge_gid)),
                    },
                "AddSceneAoiReact":
                    {
                        "Url": join_path(spool_dir, 'lr%s-gc%s-scene' % (edge_gid, edge_gid)),
                    },
                "SceneControlEvent": ["/etc/edge/mediagrabber.%s/dyn/" % edge_id]
            }
        }

        # create setupstore.conf
        if ui_control_jnode['profile'] == 'auto-70-150':
            setupstore_conf_text = get_setupstore_conf_text(edge_id)
        else:
            setupstore_conf_text = get_setupstore_conf_text(edge_id, active=False)

        # create misc.conf
        return edge_id, main_conf, setupstore_conf_text

    def serialize(self):
        edge_id, main_conf_jnode, setupstore_conf_text = self._id_main_setupstore

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_GRABCONTROL, edge_id)
        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        main_conf_abspath = join_path(edge_jnode_dir, 'main.conf')
        with open(main_conf_abspath, 'w') as main_conf:
            ujson.dump(main_conf_jnode, main_conf, indent=2)

        setupstore_conf_path = join_path(edge_jnode_dir, 'setupstore.conf')

        with open(setupstore_conf_path, 'w') as text_file:
            text_file.write(setupstore_conf_text)


class GcDeleter:
    def __init__(self, edge_gc_dirpath):
        """
        :type edge_gc_dirpath: str
        """
        self._edge_gc_dirpath = edge_gc_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: GcDeleter | None
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.GRAB_CONTROL_0)
            edge_type = Pipeline.Speed.Edge.GRAB_CONTROL_0.type

            deleter = GcDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.GRAB_CONTROL_0)
            edge_type = Pipeline.Mv.Edge.GRAB_CONTROL_0.type

            deleter = GcDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            return None

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_gc_dirpath)
