import os
import ujson

from edge.admin.ui_types import BaseCreator
from edge.path_helpers import *
from os.path import join as join_path

import edge.pipeline_helpers as helpers

from copy import deepcopy


def _mk_eventstore_react_path(ui_pid):
    """
    :type ui_pid: str
    :rtype: str
    """
    return '/var/spool/edge/system/sc{0}-es65535-fact'.format(ui_pid)


def sc_try_get_creator_ctor(ui_entry_point, used_ui_modules, used_edge_modules, avaliable_ui_modules):
    """
    :type ui_entry_point: UiIdentity
    :type used_ui_modules: set of UiIdentity
    :type used_edge_modules: set of EdgeIdentity
    :type avaliable_ui_modules: set of UiIdentity
    :rtype: None | (str, UiIdentity, dict of UiIdentity: dict) -> BaseCreator
    """
    creator_ctor = None
    ui_modules_set = set()

    # if ui_entry_point.type in [UI_TYPE_EVENTSTORE]:
    #     # restart all sc when es name changed
    #     # todo: fix this workaround
    #     helpers.restart_edge_modules(EDGE_TYPE_SPEEDCONTROL)
    #     # used_edge_modules.add(EdgeIdentity(EDGE_TYPE_SPEEDCONTROL, '0'))
    #     return None

    if ui_entry_point.type in [UI_TYPE_TRANSIT, UI_TYPE_PARKING,
                               UI_TYPE_GEOLOCATION, UI_TYPE_LANE_ENFORCEMENT,
                               UI_TYPE_EVENTSTORE, UI_TYPE_SPOT_SPEED,
                               UI_TYPE_WRONG_WAY, UI_TYPE_RED_LIGHT,
                               UI_TYPE_PIPELINE]:
        for ui_module in avaliable_ui_modules:
            if ui_module.type in [UI_TYPE_TRANSIT, UI_TYPE_PARKING,
                                  UI_TYPE_GEOLOCATION, UI_TYPE_LANE_ENFORCEMENT,
                                  UI_TYPE_EVENTSTORE, UI_TYPE_SPOT_SPEED,
                                  UI_TYPE_WRONG_WAY, UI_TYPE_RED_LIGHT]:
                ui_modules_set.add(ui_module)

        if ui_entry_point.type != UI_TYPE_PIPELINE:
            used_edge_modules.add(EdgeIdentity(EDGE_TYPE_SPEEDCONTROL, '0'))
        used_ui_modules.update(ui_modules_set)
        creator_ctor = ScCreator.create
    else:
        return None

    return lambda arg0, arg1, arg2: creator_ctor(arg0, arg1, arg2, ui_modules_set)


class ScCreator(BaseCreator):
    def __init__(self,  id_main_setupstore):
        """
        :type id_main_setupstore: str, dict, str
        """
        BaseCreator.__init__(self)
        self._id_main_setupstore = id_main_setupstore

    @staticmethod
    def create(ui_pid, ui_entry_point, ui_identity_to_jnode_map, avaliable_ui_modules):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :type avaliable_ui_modules: set of UiIdentity
        :rtype: ScCreator
        """
        if ui_entry_point.type in [UI_TYPE_TRANSIT, UI_TYPE_PARKING,
                                   UI_TYPE_GEOLOCATION, UI_TYPE_LANE_ENFORCEMENT,
                                   UI_TYPE_EVENTSTORE, UI_TYPE_SPOT_SPEED,
                                   UI_TYPE_WRONG_WAY, UI_TYPE_RED_LIGHT,
                                   UI_TYPE_PIPELINE]:
            id_main_setupstore = ScCreator._serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map,
                                                      avaliable_ui_modules)
            creator = ScCreator(id_main_setupstore)

        else:
            raise NotImplemented('pipeline not supported')

        return creator

    @staticmethod
    def _serialize(ui_pid, ui_entry_point, ui_identity_to_jnode_map, avaliable_ui_modules):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :type ui_identity_to_jnode_map: dict of UiIdentity: dict
        :type avaliable_ui_modules: set of UiIdentity
        :rtype: str, dict, str
        """
        main_conf_jnode = {}
        transit_conf_jnode = {}
        road_conf_jnode = {}
        origin_conf_jnode = {}

        edge_identity = EdgeIdentity(EDGE_TYPE_SPEEDCONTROL, '0')
        edge_group = helpers.get_edge_group(ui_pid)
        edge_id = helpers.get_edge_id(edge_group, edge_identity)
        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_SPEEDCONTROL, edge_id)

        require_plate = True
        ui_lane_enforcement_jnode = None
        ui_transit_rules_jnode = None
        ui_parking_rules_id_to_jnode_list = []
        ui_wrongway_rules_id_to_jnode_list = []
        ui_spotspeed_rules_id_to_jnode_list = []
        ui_redlight_rules_id_to_jnode_list = []
        trainer_jnode = None

        for ui_module in avaliable_ui_modules:
            if ui_module.type == UI_TYPE_LANE_ENFORCEMENT:
                ui_lane_enforcement_jnode = ui_identity_to_jnode_map[ui_module]
            elif ui_module.type == UI_TYPE_TRANSIT:
                ui_transit_rules_jnode = ui_identity_to_jnode_map[ui_module]
                require_plate = ui_transit_rules_jnode['ro'].get('require-plate', True)
                trainer_jnode = ui_transit_rules_jnode['ro'].get('trainer')
                main_conf_jnode = deepcopy(ui_transit_rules_jnode['ro'].get('main', {}))
            elif ui_module.type == UI_TYPE_EVENTSTORE:
                ui_eventstore_jnode = ui_identity_to_jnode_map[ui_module]
            elif ui_module.type == UI_TYPE_PARKING:
                ui_parking_rules_id_to_jnode_list.append((ui_module.id, ui_identity_to_jnode_map[ui_module]))
            elif ui_module.type == UI_TYPE_WRONG_WAY:
                ui_wrongway_rules_id_to_jnode_list.append((ui_module.id, ui_identity_to_jnode_map[ui_module]))
            elif ui_module.type == UI_TYPE_SPOT_SPEED:
                ui_spotspeed_rules_id_to_jnode_list.append((ui_module.id, ui_identity_to_jnode_map[ui_module]))
            elif ui_module.type == UI_TYPE_RED_LIGHT:
                ui_redlight_rules_id_to_jnode_list.append((ui_module.id, ui_identity_to_jnode_map[ui_module]))

        edge_gid = edge_identity.gid
        spool_dir = get_spool_dir(ui_pid)

        main_conf_jnode['services'] = {
            'stat': {
                'tout': 60,
                'path': join_path(spool_dir, 'zabbix', 'sc%s' % edge_gid)
            },
            'gpsd': "/etc/edge/setupstore/admin/actions/pipeline/%s/geolocation/0.json" % ui_pid,
            'db.conf': DB_JSON_FILEPATH,
            'tz.conf': UI_TIME_JNODE_FILEPATH,
            'origin': {
                'uuid': ui_eventstore_jnode['ro']['uuid'],
                'time': ui_eventstore_jnode['ro']['ctime'],
                'location': {
                    'place': ui_eventstore_jnode['rw']['origin-name']
                }
            },
            'event': {
                'endpoints': [_mk_eventstore_react_path(ui_pid)],
            },
            'react': join_path(spool_dir, 'tr%s-sc%s-track' % (edge_gid, edge_gid)),
            'systest': SYSTEST_CONF_FILEPATH
        }
        if len(ui_redlight_rules_id_to_jnode_list):
            ui_id, red_light_jnode = ui_redlight_rules_id_to_jnode_list[0]
            params_jnode = red_light_jnode['ro'].get('params', {})
            tl_id = params_jnode.get('controller', [{}])[0].get('id', 1)
            main_conf_jnode['services']['tl'] = [join_path(spool_dir, 'tl%s-sc%s-track' % (edge_gid, edge_gid)), tl_id]

        if trainer_jnode:
            main_conf_jnode['services'].update({'trainer': trainer_jnode})

        transit_conf_jnode = {
            'name': '',
            'enabled': True,
            'images': {
                'best': ui_transit_rules_jnode['rw']['args']['images']
            },
            'rules': [{
                'name': '',
                'enabled': True,
                'schedule': {
                    'days': {
                        'mo': True,
                        'tu': True,
                        'we': True,
                        'th': True,
                        'fr': True,
                        'sa': True,
                        'su': True
                    },
                    'min-time': [0, 0],
                    'max-time': [23, 59]
                },
                'limits': {
                    'require-plate': require_plate
                }
            }],
        }

        if ui_entry_point.ptype in [PTYPE_SPEED, PTYPE_LPR_MV]:
            image_mapper_dir_path = get_edge_jnode_dirpath(EDGE_TYPE_IMAGE_MAPPER, edge_id)
            image_mapper_conf_path = join_path(image_mapper_dir_path, 'main.conf')
            main_conf_jnode['services']['image-mapper.conf'] = image_mapper_conf_path

        filepath_to_content_map = [
            (join_path(edge_jnode_dir, 'main.conf'), main_conf_jnode),
            (join_path(edge_jnode_dir, 'transit.conf'), transit_conf_jnode)
        ]

        # add parking rules
        parking_rules_filepath_to_content_map = []
        for ui_id, parking_rule_jnode in ui_parking_rules_id_to_jnode_list:
            parking_rules_filepath_to_content_map.append((join_path(edge_jnode_dir, 'parking_%s.conf' % ui_id),
                                                          parking_rule_jnode['rw']['args']))

        for file_path, content in parking_rules_filepath_to_content_map:
            filepath_to_content_map.append((file_path, content))

        # # add spot-speed rules
        spotspeed_rules_filepath_to_content_map = []
        for ui_id, spotspeed_rule_jnode in ui_spotspeed_rules_id_to_jnode_list:
            spotspeed_rules_filepath_to_content_map.append((join_path(edge_jnode_dir, 'spotspeed_%s.conf' % ui_id),
                                                            spotspeed_rule_jnode['rw']['args']))

        for file_path, content in spotspeed_rules_filepath_to_content_map:
            content['videos'] = {
                'measurement': {
                    "format": "h264-mp4",
                    "max-width": 320
                }
            }
            filepath_to_content_map.append((file_path, content))

        # add wrong-way rules
        wrongway_rules_filepath_to_content_map = []
        for ui_id, wrongway_rule_jnode in ui_wrongway_rules_id_to_jnode_list:
            wrongway_rules_filepath_to_content_map.append((join_path(edge_jnode_dir, 'wrongway_%s.conf' % ui_id),
                                                           wrongway_rule_jnode['rw']['args']))

        for file_path, content in wrongway_rules_filepath_to_content_map:
            content['videos'] = {
                'measurement': {
                    "format": "h264-mp4",
                    "max-width": 320
                }
            }
            filepath_to_content_map.append((file_path, content))

        # add red-light rules
        red_light_rules_filepath_to_content_map = []
        for ui_id, red_light_rule_jnode in ui_redlight_rules_id_to_jnode_list:
            red_light_rules_filepath_to_content_map.append((join_path(edge_jnode_dir, 'redlight_%s.conf' % ui_id),
                                                            red_light_rule_jnode['rw']['args']))

        for file_path, content in red_light_rules_filepath_to_content_map:
            content['videos'] = {
                'measurement': {
                    "format": "h264-mp4",
                    "max-width": 320
                }
            }
            filepath_to_content_map.append((file_path, content))

        main_conf_jnode['detectors'] = {
            'transit.conf': [join_path(edge_jnode_dir, 'transit.conf')],
            'parking.conf': map(lambda parking_rule: parking_rule[0], parking_rules_filepath_to_content_map),
            'spot-speed.conf': map(lambda spotspeed_rule: spotspeed_rule[0], spotspeed_rules_filepath_to_content_map),
            'wrong-way.conf': map(lambda wrong_way_rule: wrong_way_rule[0], wrongway_rules_filepath_to_content_map),
            'red-light.conf': map(lambda red_light_rule: red_light_rule[0], red_light_rules_filepath_to_content_map)
        }

        if len(ui_lane_enforcement_jnode['rw']['args']['lanes']) > 0:
            road_conf_jnode = {
                'road': {
                    'lanes': ui_lane_enforcement_jnode['rw']['args']['lanes']
                },
                'scene-wh': ui_lane_enforcement_jnode['rw']['args']['scene-wh']

            }
            main_conf_jnode['services'].update({'road.conf': join_path(edge_jnode_dir, 'road.conf')})
            filepath_to_content_map.append((join_path(edge_jnode_dir, 'road.conf'), road_conf_jnode))

        # create setupstore.conf
        setupstore_conf_text = get_setupstore_conf_text(edge_id)

        return edge_id, setupstore_conf_text, filepath_to_content_map

    def serialize(self):
        edge_id, setupstore_conf_text, filepath_to_content_map = self._id_main_setupstore

        edge_jnode_dir = get_edge_jnode_dirpath(EDGE_TYPE_SPEEDCONTROL, edge_id)
        if not os.path.exists(edge_jnode_dir):
            helpers.silent_mkdir_r(edge_jnode_dir)

        for filepath, content in filepath_to_content_map:
            with open(filepath, 'w') as json_file:
                ujson.dump(content, json_file, indent=2)

        # create setupstore.conf
        setupstore_conf_filepath = join_path(edge_jnode_dir, 'setupstore.conf')
        with open(setupstore_conf_filepath, 'w') as text_file:
            text_file.write(setupstore_conf_text)


class ScDeleter:
    def __init__(self, edge_sc_dirpath):
        """
        :type edge_sc_dirpath: str
        """
        self._edge_sc_dirpath = edge_sc_dirpath

    @staticmethod
    def create(ui_pid, ui_entry_point):
        """
        :type ui_pid: str
        :type ui_entry_point: UiIdentity
        :rtype: ScDeleter
        """
        edge_group = helpers.get_edge_group(ui_pid)

        if ui_entry_point == Pipeline.Rtsp.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Rtsp.Edge.SPEEDCONTROL_0)
            edge_type = Pipeline.Rtsp.Edge.SPEEDCONTROL_0.type
            deleter = ScDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Mv.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Mv.Edge.SPEEDCONTROL_0)
            edge_type = Pipeline.Mv.Edge.SPEEDCONTROL_0.type
            deleter = ScDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        elif ui_entry_point == Pipeline.Speed.Ui.PIPELINE:
            edge_id = helpers.get_edge_id(edge_group, Pipeline.Speed.Edge.SPEEDCONTROL_0)
            edge_type = Pipeline.Speed.Edge.SPEEDCONTROL_0.type
            deleter = ScDeleter(get_edge_jnode_dirpath(edge_type, edge_id))
            return deleter

        else:
            raise NotImplemented

    def run(self):
        helpers.silent_rm_r(self._edge_sc_dirpath)
