from edge import eventlog

from .types import DetectorArgs, TransitEvidence
from ..types import IDetector
from ...model.types import BaseEvent, Event, BestWaypoint
from edge.service.tz import ITzService

from ...services.origin import IOriginService

from ...model.models import Track
from ...detectors.com.schedule import Schedule


def create(*args, **kwargs):
    """
    :rtype: IDetector
    """
    return _Detector(*args, **kwargs)


class _Detector(IDetector):
    def __init__(self, origin_service, tz_service, args):
        """
        :type tz_service: ITzService
        :type origin_service: IOriginService
        :type args: DetectorArgs
        """
        self._origin_service = origin_service
        self._scheduler = Schedule(args.schedule_args, tz_service)

        self._enabled = args.enabled
        self._require_plate = args.require_plate
        self._identity = args.identity
        self._handler = None
        IDetector.__init__(self)

    def set_handler(self, event_handler):
        """
        :type event_handler: (Event) -> None
        """
        if self._handler:
            raise NotImplementedError

        self._handler = event_handler

    def analyze(self, track, base):
        """
        :type track: Track
        :type base: BaseEvent
        """
        if not self._enabled:
            return

        waypoints_list = base.measurements[0].waypoints
        best_points = filter(
            lambda waypoint: waypoint.name == waypoint.NAME_BEST,
            waypoints_list)
        if len(best_points) != 1:
            eventlog.log_d('fail: cant find best waypoint')
            return

        best_waypoint = best_points[0]  # type: BestWaypoint
        if self._scheduler.check(best_waypoint.ts / 1000000):
            if self._require_plate and best_waypoint.vehicle.plate_absent:
                eventlog.log_d('fail: plate absent')
                return

            evidence = TransitEvidence(self._origin_service.get(best_waypoint.ts), best_waypoint.ts)
            event = Event(self._identity, base, evidence)
            self._handler(event)
