from .types import OriginServiceArgs
from edge.service.gpsd import IGpsdService

from ..model.types import Origin


class OriginError(RuntimeError):
    pass


class IOriginService:
    def __init__(self):
        pass

    def get(self, ts):
        """
        :type ts: long
        :rtype: Origin
        """
        raise NotImplementedError


def init_service(args, gpsd_service):
    """
    :type args: OriginServiceArgs
    :type gpsd_service: IGpsdService
    :rtype: IOriginService
    """
    return _OriginService(args, gpsd_service)


# noinspection PyUnusedLocal
def fini_service(service):
    """
    :type service: IOriginService | _OriginService
    """
    pass


class _OriginService:
    def __init__(self, args, gpsd_service):
        """
        :type args: OriginServiceArgs
        :type gpsd_service: IGpsdService
        """
        self._gpsd_service = gpsd_service
        self._args = args

    def get(self, ts):
        """
        :type ts: long
        :rtype: Origin
        """
        geolocation = self._gpsd_service.get_geodata(ts)
        return Origin(self._args.uuid, self._args.time,
                      self._args.location, geolocation)
